@@ -6,6 +6,7 @@ import numpy as np
 from functools import partial
 from matplotlib.pyplot import Artist, Axes
 from matplotlib.backend_bases import Event
+import uuid
 from typing import Tuple
 
 
@@ -58,11 +59,12 @@ class Lines(Tool):
         if 'marker' not in kwargs:
             kwargs['marker'] = 'o'
         line, = self._ax.plot(xpos, ypos, **kwargs)
+        line.id = str(uuid.uuid1())
         self.lines.append(line)
         self._artist_counter += 1
 
     def _on_motion_notify(self, event: Event):
-        self._move_vertex(event=event, ind=-1, line=self.lines[-1])
+        self._move_vertex(event=event, ind=-1, artist=self.lines[-1])
 
     def _after_line_creation(self, event: Event):
         self._connect({'motion_notify_event': self._on_motion_notify})
@@ -96,13 +98,14 @@ class Lines(Tool):
     def _finalize_line(self, event: Event):
         self.lines[-1].set_picker(5.0)
         if self.on_create is not None:
-            self.on_create(event)
+            self.on_create({'event': event, 'artist': self.lines[-1]})
         self._draw()
 
-    def _remove_line(self, line: Artist):
+    def _remove_line(self, line: Artist, draw: bool = True):
         line.remove()
         self.lines.remove(line)
-        self._draw()
+        if draw:
+            self._draw()
 
     def _on_pick(self, event: Event):
         if self._get_active_tool():
@@ -113,16 +116,20 @@ class Lines(Tool):
             self._pick_lock = True
             self._grab_vertex(event)
             if self.on_vertex_press is not None:
-                self.on_vertex_press(event)
+                self.on_vertex_press({
+                    'event': event,
+                    'ind': self._moving_vertex_index,
+                    'artist': self._moving_vertex_artist
+                })
         elif event.mouseevent.button == 2:
             self._remove_line(event.artist)
             if self.on_remove is not None:
-                self.on_remove(event)
+                self.on_remove({'event': event, 'artist': event.artist})
         elif event.mouseevent.button == 3:
             self._pick_lock = True
             self._grab_line(event)
             if self.on_drag_press is not None:
-                self.on_drag_press(event)
+                self.on_drag_press({'event': event, 'artist': self._grab_artist})
 
     def _grab_vertex(self, event: Event):
         self._connect({
@@ -134,19 +141,22 @@ class Lines(Tool):
         self._moving_vertex_artist = event.artist
 
     def _on_vertex_motion(self, event: Event):
-        self._move_vertex(event=event,
-                          ind=self._moving_vertex_index,
-                          line=self._moving_vertex_artist)
+        event_dict = {
+            'event': event,
+            'ind': self._moving_vertex_index,
+            'artist': self._moving_vertex_artist
+        }
+        self._move_vertex(**event_dict)
         if self.on_vertex_move is not None:
-            self.on_vertex_move(event)
+            self.on_vertex_move(event_dict)
 
-    def _move_vertex(self, event: Event, ind: int, line: Artist):
+    def _move_vertex(self, event: Event, ind: int, artist: Artist):
         if event.inaxes != self._ax:
             return
-        new_data = line.get_data()
+        new_data = artist.get_data()
         new_data[0][ind] = event.xdata
         new_data[1][ind] = event.ydata
-        line.set_data(new_data)
+        artist.set_data(new_data)
         self._draw()
 
     def _grab_line(self, event: Event):
@@ -155,18 +165,19 @@ class Lines(Tool):
             'button_release_event': partial(self._release_line, kind='grab')
         })
 
-        self._grab_artist = event.artist
+        self._grab_artist = getattr(event.artist, '_line', event.artist)
         self._grab_mouse_origin = event.mouseevent.xdata, event.mouseevent.ydata
         self._grab_artist_origin = self._grab_artist.get_data()
 
-    def _move_line(self, event: Event):
+    def _move_line(self, event: Event, draw: bool = True):
         if event.inaxes != self._ax:
             return
         dx = event.xdata - self._grab_mouse_origin[0]
         dy = event.ydata - self._grab_mouse_origin[1]
         self._grab_artist.set_data(
             (self._grab_artist_origin[0] + dx, self._grab_artist_origin[1] + dy))
-        self._draw()
+        if draw:
+            self._draw()
 
     def _release_line(self, event: Event, kind: str):
         self._disconnect(['motion_notify_event', 'button_release_event'])
